package be.ac.ulb.code.wit;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.Set;

import javax.xml.namespace.QName;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPMessage;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import javax.xml.ws.handler.MessageContext;
import javax.xml.ws.handler.soap.SOAPHandler;
import javax.xml.ws.handler.soap.SOAPMessageContext;

class SOAPDebugger implements SOAPHandler<SOAPMessageContext> {
    public SOAPDebugger() {
	}

	public void close(MessageContext messagecontext) {
	}

	public Set<QName> getHeaders() {
		return null;
	}
	
	/**
	 * Repeat the specified text a number of times.
	 * @param text The text to be repeated
	 * @param count The number of repetitions of the text
	 * @return A String that is {@value count} times the repetition of {@value text}
	 */
	private String repeat(String text, int count) {
		StringBuilder b = new StringBuilder(text.length() * count);
		
		while(count-- != 0) {
			b.append(text);
		}
		
		return b.toString();
	}
	
	/**
	 * Get a string to be used as an header prior to displaying message content
	 * @param messagecontext The message to be logged
	 * @param what In what context (fault, transmission) the message is being processed
	 * @return A string providing some context about the message
	 */
	private String getDebugHeader(SOAPMessageContext messagecontext, String what) {
		Boolean outbound = (Boolean) messagecontext.get(MessageContext.MESSAGE_OUTBOUND_PROPERTY);
		String s = outbound ? "Client" : "Server";
		String t = outbound ? "Server" : "Client";
		return "Handling " + what + " in " + s + " --> " + t + " message\n" + 
			repeat("-", 20) +"\n";
	}

	/**
	 * Pretty-format an XML string.
	 * See http://stackoverflow.com/questions/139076/how-to-pretty-print-xml-from-java for details.
	 * 
	 * @param input The XML string to be formatted.
	 * @param indent The number of spaces for each level of indentation
	 * @return A string containing properly indented XML.
	 */
	public static String prettyFormatXML(String input, int indent) {
	    try {
	        Source xmlInput = new StreamSource(new StringReader(input));
	        StringWriter stringWriter = new StringWriter();
	        StreamResult xmlOutput = new StreamResult(stringWriter);
	        
	        TransformerFactory transformerFactory = TransformerFactory.newInstance();
	        transformerFactory.setAttribute("indent-number", indent);
	        
	        Transformer transformer = transformerFactory.newTransformer(); 
	        transformer.setOutputProperty(OutputKeys.INDENT, "yes");
	        transformer.transform(xmlInput, xmlOutput);
	        return xmlOutput.getWriter().toString();
	    } catch (Exception e) {
	        throw new RuntimeException(e); 
	    }
	}

	/**
	 * Displays the content of a SOAP message in a human-readable way.
	 * 
	 * @param message The SOAP message to the displayed.
	 * @throws SOAPException
	 * @throws IOException
	 */
	private void printSoapMessage(SOAPMessage message) throws SOAPException, IOException {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		message.writeTo(baos);
		
		String xmlMessage = new String(baos.toByteArray());
		
		System.out.println(prettyFormatXML(xmlMessage, 2));
		System.out.println();
	}
	
	public boolean handleFault(SOAPMessageContext messagecontext) {
		System.out.println(getDebugHeader(messagecontext, "fault"));
		try {
			printSoapMessage(messagecontext.getMessage());
		} catch (SOAPException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	    return true;
	}
	
	public boolean handleMessage(SOAPMessageContext messagecontext) {
		System.out.println(getDebugHeader(messagecontext, "transmission"));
		
		try {
			printSoapMessage(messagecontext.getMessage());
			System.out.println();
		} catch (SOAPException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		return true;
	}
}
